use clap::{ArgMatches, Command};
use color_eyre::Result;

mod brute_force_words;
mod extract_words;

pub(crate) fn command_definition() -> Command {
    Command::new("experiment")
        .subcommand_required(true)
        .about("A collection of utilities and experiments.")
        .subcommand(brute_force_words::command_definition())
        .subcommand(extract_words::command_definition())
}

#[tracing::instrument(skip_all)]
pub(crate) async fn run(ctx: sdk::Context, matches: &ArgMatches) -> Result<()> {
    match matches.subcommand() {
        // It's fine to block here, as this is the only thing that's executing on the runtime.
        // The other option with `spawn_blocking` would require setting up values to be Send+Sync.
        Some(("brute-force-words", sub_matches)) => brute_force_words::run(ctx, sub_matches),
        Some(("extract-words", sub_matches)) => extract_words::run(ctx, sub_matches).await,
        _ => unreachable!(
            "clap is configured to require a subcommand, and they're all handled above"
        ),
    }
}
